/*
 * FCKeditor - The text editor for Internet - http://www.fckeditor.net
 * Copyright (C) 2003-2007 Frederico Caldeira Knabben
 *
 * == BEGIN LICENSE ==
 *
 * Licensed under the terms of any of the following licenses at your
 * choice:
 *
 *  - GNU General Public License Version 2 or later (the "GPL")
 *    http://www.gnu.org/licenses/gpl.html
 *
 *  - GNU Lesser General Public License Version 2.1 or later (the "LGPL")
 *    http://www.gnu.org/licenses/lgpl.html
 *
 *  - Mozilla Public License Version 1.1 or later (the "MPL")
 *    http://www.mozilla.org/MPL/MPL-1.1.html
 *
 * == END LICENSE ==
 *
 * This is the code behind of the connector.aspx page used by the 
 * File Browser.
 */

using System;
using System.Text;
using System.Web;

namespace FredCK.FCKeditorV2.FileBrowser
{
    public class Config : System.Web.UI.UserControl
    {
        private const string DEFAULT_USER_FILES_PATH = "/userfiles/";

        private string sUserFilesDirectory;

        public bool Enabled;
        public string UserFilesPath;
        public string UserFilesAbsolutePath;
        public bool ForceSingleExtension;
        public string[] AllowedTypes;
        public string[] HtmlExtensions;
        public TypeConfigList TypeConfig;

        #region ZhuBo's Modification
        /// <summary>
        /// Pattern of file folder. Separate by slash "/". At the End without slash.
        /// 
        /// Can be set in the Web.config file using the appSettings section.
        /// For example:
        ///     &lt;add key="FCKeditor:FolderPattern" value="%y/%m/%d/"/&gt;
        /// 
        /// Parameters
        ///     %u      user's identifier
        ///     %y      Year
        ///     %m      Month
        ///     %d      Day
        /// 
        /// e.g
        ///     %y/%m-%d
        /// </summary>
        public string FolderPattern;

        /// <summary>
        /// Pattern of filename.
        /// 
        /// Can be set in the Web.config file using the appSettings section.
        /// For example:
        ///     &lt;add key="FCKeditor:FilenamePattern" value="%guid.%extl"/&gt;
        /// 
        /// Parameters
        ///     %guid   Guid
        ///     %fnl    Uploaded File's Name without extension
        ///     %fnu    Uploaded File's Name in upper case
        ///     %extl   File Extension
        ///     %extu   File Extension in upper case
        /// 
        /// e.g
        ///     %fnl.%extl
        ///     %guid.%extl
        /// </summary>
        public string FilenamePattern;

        /// <summary>
        /// The domain name of uploaded files.
        /// 
        /// Can be set in the Web.config file using the appSettings section.
        /// For example:
        ///     &lt;add key="FCKeditor:UploadedFilesDomain" value="http://localhost/"/&gt;
        /// </summary>
        public string UploadedFilesDomain;

        /// <summary>
        /// Max width or height of uploaded images in pixel.
        /// 
        /// Can be set in the Web.config file using the appSettings section.
        /// For example:
        ///     &lt;add key="FCKeditor:MaxWidthOfUploadedImg" value="200"/&gt;
        /// </summary>
        public Int32 MaxWidthOfUploadedImg;

        /// <summary>
        /// Identifier for logind user.
        /// It's user's id usually.
        /// </summary>
        public String LoginedUserIdentifier;
        #endregion

        public Config()
        { }

        private void DefaultSettings()
        {
            // Initialize all default settings.

            #region ZhuBo's Modification
            FolderPattern = String.Empty;
            FilenamePattern = "%fnl.%extl";
            UploadedFilesDomain = String.Empty;
            MaxWidthOfUploadedImg = 0;
            LoginedUserIdentifier = String.Empty;
            #endregion

            Enabled = false;
            UserFilesPath = "/userfiles/";
            UserFilesAbsolutePath = "";
            ForceSingleExtension = true;
            AllowedTypes = new string[] { "File", "Image", "Flash", "Media" };
            HtmlExtensions = new string[] { "html", "htm", "xml", "xsd", "txt", "js" };

            TypeConfig = new TypeConfigList((FileWorkerBase)this.Page);

            TypeConfig["File"].AllowedExtensions = new string[] { "7z", "aiff", "asf", "avi", "bmp", "csv", "doc", "fla", "flv", "gif", "gz", "gzip", "jpeg", "jpg", "mid", "mov", "mp3", "mp4", "mpc", "mpeg", "mpg", "ods", "odt", "pdf", "png", "ppt", "pxd", "qt", "ram", "rar", "rm", "rmi", "rmvb", "rtf", "sdc", "sitd", "swf", "sxc", "sxw", "tar", "tgz", "tif", "tiff", "txt", "vsd", "wav", "wma", "wmv", "xls", "xml", "zip" };
            TypeConfig["File"].DeniedExtensions = new string[] { };
            TypeConfig["File"].FilesPath = "%UserFilesPath%file/";
            TypeConfig["File"].FilesAbsolutePath = (UserFilesAbsolutePath == "" ? "" : "%UserFilesAbsolutePath%file/");
            TypeConfig["File"].QuickUploadPath = "%UserFilesPath%";
            TypeConfig["File"].QuickUploadAbsolutePath = "%UserFilesAbsolutePath%";

            TypeConfig["Image"].AllowedExtensions = new string[] { "bmp", "gif", "jpeg", "jpg", "png" };
            TypeConfig["Image"].DeniedExtensions = new string[] { };
            TypeConfig["Image"].FilesPath = "%UserFilesPath%image/";
            TypeConfig["Image"].FilesAbsolutePath = (UserFilesAbsolutePath == "" ? "" : "%UserFilesAbsolutePath%image/");
            TypeConfig["Image"].QuickUploadPath = "%UserFilesPath%";
            TypeConfig["Image"].QuickUploadAbsolutePath = "%UserFilesAbsolutePath%";

            TypeConfig["Flash"].AllowedExtensions = new string[] { "swf", "flv" };
            TypeConfig["Flash"].DeniedExtensions = new string[] { };
            TypeConfig["Flash"].FilesPath = "%UserFilesPath%flash/";
            TypeConfig["Flash"].FilesAbsolutePath = (UserFilesAbsolutePath == "" ? "" : "%UserFilesAbsolutePath%flash/");
            TypeConfig["Flash"].QuickUploadPath = "%UserFilesPath%";
            TypeConfig["Flash"].QuickUploadAbsolutePath = "%UserFilesAbsolutePath%";

            TypeConfig["Media"].AllowedExtensions = new string[] { "aiff", "asf", "avi", "bmp", "fla", "flv", "gif", "jpeg", "jpg", "mid", "mov", "mp3", "mp4", "mpc", "mpeg", "mpg", "png", "qt", "ram", "rm", "rmi", "rmvb", "swf", "tif", "tiff", "wav", "wma", "wmv" };
            TypeConfig["Media"].DeniedExtensions = new string[] { };
            TypeConfig["Media"].FilesPath = "%UserFilesPath%media/";
            TypeConfig["Media"].FilesAbsolutePath = (UserFilesAbsolutePath == "" ? "" : "%UserFilesAbsolutePath%media/");
            TypeConfig["Media"].QuickUploadPath = "%UserFilesPath%";
            TypeConfig["Media"].QuickUploadAbsolutePath = "%UserFilesAbsolutePath%";
        }

        internal void LoadConfig()
        {
            DefaultSettings();

            // Call the setConfig() function for the configuration file (config.ascx).
            SetConfig();

            #region ZhuBo's Modification
            // Look for possible FolderPattern override options.

            // Session
            string folderPattern = Session["FCKeditor:FolderPattern"] as string;

            // Application
            if (folderPattern == null || folderPattern.Length == 0)
                folderPattern = Application["FCKeditor:FolderPattern"] as string;

            // Web.config file.
            if (folderPattern == null || folderPattern.Length == 0)
                folderPattern = System.Configuration.ConfigurationSettings.AppSettings["FCKeditor:FolderPattern"];

            // config.asxc
            if (folderPattern == null || folderPattern.Length == 0)
                folderPattern = this.FolderPattern;

            // Make sure that the FolderPattern starts with slash, while not ends with slash ("/").
            if (folderPattern.StartsWith("/"))
                folderPattern = folderPattern.Substring(1);

            if (!folderPattern.EndsWith("/") && !String.IsNullOrEmpty(folderPattern))
                folderPattern += "/";


            this.FolderPattern = folderPattern;

            // Look for possible FilenamePattern override options.

            // Session
            string filenamePattern = Session["FCKeditor:FilenamePattern"] as string;

            // Application
            if (filenamePattern == null || filenamePattern.Length == 0)
                filenamePattern = Application["FCKeditor:FilenamePattern"] as string;

            // Web.config file.
            if (filenamePattern == null || filenamePattern.Length == 0)
                filenamePattern = System.Configuration.ConfigurationSettings.AppSettings["FCKeditor:FilenamePattern"];

            // config.asxc
            if (filenamePattern == null || filenamePattern.Length == 0)
                filenamePattern = this.FilenamePattern;

            this.FilenamePattern = filenamePattern;

            // Look for possible UploadedFilesDomain override options.

            // Session
            string uploadedFilesDomain = Session["FCKeditor:UploadedFilesDomain"] as string;

            // Application
            if (uploadedFilesDomain == null || uploadedFilesDomain.Length == 0)
                uploadedFilesDomain = Application["FCKeditor:UploadedFilesDomain"] as string;

            // Web.config file.
            if (uploadedFilesDomain == null || uploadedFilesDomain.Length == 0)
                uploadedFilesDomain = System.Configuration.ConfigurationSettings.AppSettings["FCKeditor:UploadedFilesDomain"];

            // config.asxc
            if (uploadedFilesDomain == null || uploadedFilesDomain.Length == 0)
                uploadedFilesDomain = this.UploadedFilesDomain;

            // Make sure that the UploadedFilesDomain not ends with slash ("/")
            if (uploadedFilesDomain.EndsWith("/"))
                uploadedFilesDomain = uploadedFilesDomain.Substring(0, uploadedFilesDomain.Length - 1);

            this.UploadedFilesDomain = uploadedFilesDomain;

            // Look for possible MaxWidthOfUploadedImg override options.

            // Session
            string maxWidthOfUploadedImg = Session["FCKeditor:MaxWidthOfUploadedImg"] as string;

            // Application
            if (maxWidthOfUploadedImg == null || maxWidthOfUploadedImg.Length == 0)
                maxWidthOfUploadedImg = Application["FCKeditor:MaxWidthOfUploadedImg"] as string;

            // Web.config file.
            if (maxWidthOfUploadedImg == null || maxWidthOfUploadedImg.Length == 0)
                maxWidthOfUploadedImg = System.Configuration.ConfigurationSettings.AppSettings["FCKeditor:MaxWidthOfUploadedImg"];

            // config.asxc
            if (maxWidthOfUploadedImg == null || maxWidthOfUploadedImg.Length == 0)
                maxWidthOfUploadedImg = this.MaxWidthOfUploadedImg.ToString();

            this.MaxWidthOfUploadedImg = Convert.ToInt32(maxWidthOfUploadedImg);

            // Look for possible LoginedUserIdentifier override options.

            // Session
            string loginedUserIdentifier = Session["FCKeditor:LoginedUserIdentifier"] as string;

            // config.asxc
            if (!String.IsNullOrEmpty(loginedUserIdentifier))
                loginedUserIdentifier = this.LoginedUserIdentifier;
            #endregion

            // Look for possible UserFilesPath override options.

            // Session
            string userFilesPath = Session["FCKeditor:UserFilesPath"] as string;

            // Application
            if (userFilesPath == null || userFilesPath.Length == 0)
                userFilesPath = Application["FCKeditor:UserFilesPath"] as string;

            // Web.config file.
            if (userFilesPath == null || userFilesPath.Length == 0)
                userFilesPath = System.Configuration.ConfigurationSettings.AppSettings["FCKeditor:UserFilesPath"];

            // config.asxc
            if (userFilesPath == null || userFilesPath.Length == 0)
                userFilesPath = this.UserFilesPath;

            if (userFilesPath == null || userFilesPath.Length == 0)
                userFilesPath = DEFAULT_USER_FILES_PATH;

            // Check that the user path ends with slash ("/")
            if (!userFilesPath.EndsWith("/"))
                userFilesPath += "/";

            userFilesPath = this.ResolveUrl(userFilesPath);

            this.UserFilesPath = userFilesPath;
        }

        /// <summary>
        /// The absolution path (server side) of the user files directory. It 
        /// is based on the <see cref="FileWorkerBase.UserFilesPath"/>.
        /// </summary>
        internal string UserFilesDirectory
        {
            get
            {
                if (sUserFilesDirectory == null)
                {
                    if (this.UserFilesAbsolutePath.Length > 0)
                    {
                        sUserFilesDirectory = this.UserFilesAbsolutePath;
                        sUserFilesDirectory = sUserFilesDirectory.TrimEnd('\\', '/') + '/';
                    }
                    else
                    {
                        // Get the local (server) directory path translation.
                        sUserFilesDirectory = Server.MapPath(this.UserFilesPath);
                    }
                }
                return sUserFilesDirectory;
            }
        }

        public virtual void SetConfig()
        { }

        internal bool CheckIsTypeAllowed(string typeName)
        {
            return (System.Array.IndexOf(this.AllowedTypes, typeName) >= 0);
        }

        internal bool CheckIsNonHtmlExtension(string extension)
        {
            return (this.HtmlExtensions.Length == 0 || !Util.ArrayContains(this.HtmlExtensions, extension, System.Collections.CaseInsensitiveComparer.DefaultInvariant));
        }
    }
}
