/*
 * FCKeditor - The text editor for Internet - http://www.fckeditor.net
 * Copyright (C) 2003-2007 Frederico Caldeira Knabben
 *
 * == BEGIN LICENSE ==
 *
 * Licensed under the terms of any of the following licenses at your
 * choice:
 *
 *  - GNU General Public License Version 2 or later (the "GPL")
 *    http://www.gnu.org/licenses/gpl.html
 *
 *  - GNU Lesser General Public License Version 2.1 or later (the "LGPL")
 *    http://www.gnu.org/licenses/lgpl.html
 *
 *  - Mozilla Public License Version 1.1 or later (the "MPL")
 *    http://www.mozilla.org/MPL/MPL-1.1.html
 *
 * == END LICENSE ==
 *
 * Base class used by the FileBrowserConnector and Uploader.
 */

using System;
using System.Web;
using System.Text.RegularExpressions;

namespace FredCK.FCKeditorV2.FileBrowser
{
    public abstract class FileWorkerBase : System.Web.UI.Page
    {
        public Config Config;

        protected void FileUpload(string resourceType, string currentFolder, bool isQuickUpload)
        {
            HttpPostedFile oFile = Request.Files["NewFile"];

            string sFileName = "";

            if (oFile == null)
            {
                this.SendFileUploadResponse(202, isQuickUpload);
                return;
            }

            // Map the virtual path to the local server path.
            string sServerDir = this.ServerMapFolder(resourceType, currentFolder, isQuickUpload);

            // Get the uploaded file name.
            sFileName = System.IO.Path.GetFileName(oFile.FileName);
            sFileName = this.SanitizeFileName(sFileName);

            string sExtension = System.IO.Path.GetExtension(oFile.FileName);
            sExtension = sExtension.TrimStart('.');

            if (!this.Config.TypeConfig[resourceType].CheckIsAllowedExtension(sExtension))
            {
                this.SendFileUploadResponse(202, isQuickUpload);
                return;
            }

            if (this.Config.CheckIsNonHtmlExtension(sExtension) && !this.CheckNonHtmlFile(oFile))
            {
                this.SendFileUploadResponse(202, isQuickUpload);
                return;
            }

            #region ZhuBo's Modification
            String sPath = FileFolderOnTheBasisOfPattern(sServerDir);
            sServerDir = sServerDir + sPath;
            sExtension = FileExtensionOnTheBasisOfPattern(sExtension);
            sFileName = FileNameOnTheBasisOfPattern(sFileName) + "." + sExtension;

            String uploadedFilePath = String.Empty;
            #endregion

            int iErrorNumber = 0;
            int iCounter = 0;

            while (true)
            {
                string sFilePath = System.IO.Path.Combine(sServerDir, sFileName);

                if (System.IO.File.Exists(sFilePath))
                {
                    iCounter++;
                    sFileName =
                        System.IO.Path.GetFileNameWithoutExtension(oFile.FileName) +
                        "(" + iCounter + ")." +
                        sExtension;

                    iErrorNumber = 201;
                }
                else
                {
                    oFile.SaveAs(sFilePath);

                    #region ZhuBo's Modification
                    uploadedFilePath = sFilePath;
                    #endregion

                    break;
                }
            }

            TypeConfig typeConfig = this.Config.TypeConfig[resourceType];

            string sFileUrl = isQuickUpload ? typeConfig.GetQuickUploadPath() : typeConfig.GetFilesPath();

            #region ZhuBo's Modification
            sFileUrl = this.Config.UploadedFilesDomain + sFileUrl + sPath;

            //Generate thumbnail when MaxWidthOfUploadedImg > 0 and resource type is "Image".
            if (this.Config.MaxWidthOfUploadedImg > 0 && resourceType == "Image")
            {
                if (sExtension.ToLower() != "gif")
                {
                    this.MakeThumbnail(uploadedFilePath, this.Config.MaxWidthOfUploadedImg);
                }
            }
            #endregion

            sFileUrl += sFileName;

            this.SendFileUploadResponse(iErrorNumber, isQuickUpload, sFileUrl, sFileName);
        }

        #region ZhuBo's Modification
        /// <summary>
        /// Computing file folder on the basis of the pattern in the configuration, returns the result.
        /// </summary>
        /// <param name="sServerDir">The folder's absolute path which storing the uploaded files.</param>
        /// <returns></returns>
        private String FileFolderOnTheBasisOfPattern(String sServerDir)
        {
            String path = this.Config.FolderPattern;

            if (!String.IsNullOrEmpty(path))
            {
                DateTime date = DateTime.Now;

                path = path.Replace("%u", this.Config.LoginedUserIdentifier)
                    .Replace("%y", date.Year.ToString())
                    .Replace("%m", date.Month.ToString())
                    .Replace("%d", date.Day.ToString());

                if (!System.IO.Directory.Exists(sServerDir + path))
                {
                    System.IO.Directory.CreateDirectory(sServerDir + path);
                }
            }

            return path;
        }

        /// <summary>
        /// Computing uploaded file's filename on the basis of the pattern in the configuration.
        /// </summary>
        /// <param name="fileName">Uploaded file's name without extension.</param>
        /// <returns></returns>
        private String FileNameOnTheBasisOfPattern(String fileName)
        {
            fileName = fileName.Split('.')[0];
            String fileNamePattern = this.Config.FilenamePattern.Split('.')[0];

            return fileNamePattern.Replace("%guid", Guid.NewGuid().ToString())
                .Replace("%fnl", fileName.ToLower())
                .Replace("%fnu", fileName.ToUpper());
        }

        /// <summary>
        /// Computing uploaded file's extension on the basis of the pattern in the configuration.
        /// </summary>
        /// <param name="fileExtension">Uploaded file's extension.</param>
        /// <returns></returns>
        private String FileExtensionOnTheBasisOfPattern(String fileExtension)
        {
            String fileExtensionPattern = this.Config.FilenamePattern.Split('.')[1];

            return fileExtensionPattern.Replace("%extl", fileExtension.ToLower())
                .Replace("%extu", fileExtension.ToUpper());
        }

        /// <summary>
        /// ָȵͼ
        /// </summary>
        /// <param name="originalImagePath">Դͼ··</param>
        /// <param name="maxWidth">ͼ߳</param>
        public void MakeThumbnail(String originalImagePath, Int32 maxWidth)
        {
            if (System.IO.File.Exists(originalImagePath))
            {
                String tempFile = originalImagePath + "Temp";

                //ԭͼļΪʱļ
                System.IO.File.Move(originalImagePath, tempFile);

                System.Drawing.Image originalImage = System.Drawing.Image.FromFile(tempFile);
                Int32 towidth = originalImage.Width;
                Int32 toheight = originalImage.Height;
                Int32 x = 0;
                Int32 y = 0;
                Int32 ow = originalImage.Width;
                Int32 oh = originalImage.Height;

                //ָ߰
                //ֻеԭͼȴָʱŽ
                if (originalImage.Width > maxWidth)
                {
                    towidth = maxWidth;
                    toheight = originalImage.Height * maxWidth / originalImage.Width;
                }

                //Utility.LogInfo("Make Thunbnail", "mode = " + mode.ToString() + ", width = " + width + ", height = " + height + ", towidth = " + towidth + ", toheight = " + toheight + ", x = " + x + ", y = " + y + ", ow = " + ow + ", oh = " + oh);

                //½һbmpͼƬ
                System.Drawing.Image bitmap = new System.Drawing.Bitmap(towidth, toheight);

                //½һ
                System.Drawing.Graphics g = System.Drawing.Graphics.FromImage(bitmap);

                //øֵ
                g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.High;

                //ø,ٶȳƽ̶
                g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;

                //ջ͸ɫ
                g.Clear(System.Drawing.Color.Transparent);

                //ָλòҰָСԭͼƬָ
                g.DrawImage(originalImage,
                    new System.Drawing.Rectangle(0, 0, towidth, toheight),
                    new System.Drawing.Rectangle(x, y, ow, oh),
                    System.Drawing.GraphicsUnit.Pixel);

                try
                {
                    //jpgʽͼ
                    bitmap.Save(originalImagePath, System.Drawing.Imaging.ImageFormat.Jpeg);
                }
                catch (System.Exception e)
                {
                    this.SendFileUploadResponse(1, true, "", "", e.ToString());
                }
                finally
                {
                    originalImage.Dispose();
                    bitmap.Dispose();
                    g.Dispose();

                    //ɾʱļ
                    System.IO.File.Delete(tempFile);
                }
            }
            else
            {
                this.SendFileUploadResponse(1, true, "", "", "Original image not found");
            }
        }
        #endregion

        private void SendFileUploadResponse(int errorNumber, bool isQuickUpload)
        {
            this.SendFileUploadResponse(errorNumber, isQuickUpload, "", "", "");
        }

        private void SendFileUploadResponse(int errorNumber, bool isQuickUpload, string fileUrl, string fileName)
        {
            this.SendFileUploadResponse(errorNumber, isQuickUpload, fileUrl, fileName, "");
        }

        protected void SendFileUploadResponse(int errorNumber, bool isQuickUpload, string fileUrl, string fileName, string customMsg)
        {
            Response.Clear();

            Response.Write("<script type=\"text/javascript\">");

            // Minified version of the document.domain automatic fix script.
            // The original script can be found at _dev/domain_fix_template.js
            #region ZhuBo's Modification
            // Response.Write( @"(function(){var d=document.domain;while (true){try{var A=window.top.opener.document.domain;break;}catch(e) {};d=d.replace(/.*?(?:\.|$)/,'');if (d.length==0) break;try{document.domain=d;}catch (e){break;}}})();" );
            #endregion

            if (isQuickUpload)
                Response.Write("window.parent.OnUploadCompleted(" + errorNumber + ",'" + fileUrl.Replace("'", "\\'") + "','" + fileName.Replace("'", "\\'") + "','" + customMsg.Replace("'", "\\'") + "') ;");
            else
                Response.Write("window.parent.frames['frmUpload'].OnUploadCompleted(" + errorNumber + ",'" + fileName.Replace("'", "\\'") + "') ;");

            Response.Write("</script>");

            Response.End();
        }

        protected string ServerMapFolder(string resourceType, string folderPath, bool isQuickUpload)
        {
            TypeConfig typeConfig = this.Config.TypeConfig[resourceType];

            // Get the resource type directory.
            string sResourceTypePath = isQuickUpload ? typeConfig.GetQuickUploadDirectory() : typeConfig.GetFilesDirectory();

            // Ensure that the directory exists.
            Util.CreateDirectory(sResourceTypePath);

            // Return the resource type directory combined with the required path.
            return System.IO.Path.Combine(sResourceTypePath, folderPath.TrimStart('/'));
        }


        // Do a cleanup of the folder name to avoid possible problems
        protected string SanitizeFolderName(string folderName)
        {
            // Remove . \ / | : ? * " < >
            return Regex.Replace(folderName, @"[.\\/|:?*""<>\p{C}]", "_", RegexOptions.None);
        }

        // Do a cleanup of the file name to avoid possible problems
        private string SanitizeFileName(string fileName)
        {
            // Replace dots in the name with underscores (only one dot can be there... security issue).
            if (Config.ForceSingleExtension)
                fileName = Regex.Replace(fileName, @"\.(?![^.]*$)", "_", RegexOptions.None);

            // Remove \ / | : ? * " < >
            return Regex.Replace(fileName, @"[\\/|:?*""<>\p{C}]", "_", RegexOptions.None);
        }

        private bool CheckNonHtmlFile(HttpPostedFile file)
        {
            byte[] buffer = new byte[1024];
            file.InputStream.Read(buffer, 0, 1024);

            string firstKB = System.Text.ASCIIEncoding.ASCII.GetString(buffer);

            if (Regex.IsMatch(firstKB, @"<!DOCTYPE\W*X?HTML", RegexOptions.IgnoreCase | RegexOptions.Singleline))
                return false;

            if (Regex.IsMatch(firstKB, @"<(?:body|head|html|img|pre|script|table|title)", RegexOptions.IgnoreCase | RegexOptions.Singleline))
                return false;

            //type = javascript
            if (Regex.IsMatch(firstKB, @"type\s*=\s*[\'""]?\s*(?:\w*/)?(?:ecma|java)", RegexOptions.IgnoreCase | RegexOptions.Singleline))
                return false;

            //href = javascript
            //src = javascript
            //data = javascript
            if (Regex.IsMatch(firstKB, @"(?:href|src|data)\s*=\s*[\'""]?\s*(?:ecma|java)script:", RegexOptions.IgnoreCase | RegexOptions.Singleline))
                return false;

            //url(javascript
            if (Regex.IsMatch(firstKB, @"url\s*\(\s*[\'""]?\s*(?:ecma|java)script:", RegexOptions.IgnoreCase | RegexOptions.Singleline))
                return false;

            return true;
        }
    }
}